﻿// #AdvancedSearch[En].jsx

/*
   
v.10072022  

=== Clipboard text in the search line ===

Search in the layout of the text placed in the buffer, a few words, or a few paragraphs, it does not matter.

Changing spaces to spaces, changing the type of quotation marks, removing duplicate spaces, changing a hyphen to a minus or dash is
the main reason that there is no ease of searching in the indesign of the text taken to the buffer in the Word editor or in a PDF file.

When preparing the text for layout, repeated spaces were removed, the hyphen in the sequence of characters "space-hyphen-space" between words was replaced with a dash or minus, the type of quotation marks was changed.
Some gaps have become variations. This, of course, is good. But the convenience of searching for text has disappeared. It is impossible to take the text in the Word editor to the buffer and immediately find it in the layout.

This script allows you to search in the layout for the text taken into the buffer, turning a blind eye to the following cases:
= repeated spaces, and the fact that they could become variations;
= hyphens/minus/dashes surrounded by hyphens are considered the same characters;
= if there is a quotation mark, then it can be any.
= the difference in the number of repeated line feeds is not considered a discrepancy between the layout and the source text.

Punctuation marks — comma, dot, etc. should be the same as in the text submitted for layout. There can be no options here.

This solution allows you to buffer raw text from a Word editor or notepad, or from a PDF file, and search for it in the layout.
Moreover, only to take, then — the preparation of the text for such a search, without the control of variations, quotation marks, which is used as a dash,
and placing this line in the "Find text:" field in the GREP panel will make this script.

The buffer text is changed so that the spaces in place of spaces and the different type of quotation marks are not an obstacle to search.
- in the text from the buffer, spaces are replaced with \h — grep-the search code for spaces and tabs.
- the type of quotation marks can also be different, a dot is put in their place as the code of any sign.
- the text prepared for the search is placed in the grep search field.

The search is performed from the insertion point of the cursor to the end of the material, if the cursor is in the text, or in the selected area of the material.
The found text is displayed in the middle of the screen at a scale of 150%.

Script for CS6+ versions.

Instructions with slightly different accents on the use of this program are in the accompanying PDF file.

-----------------------------------------------
Mikhail Ivanyushin
https://shop.dotextok.ru /en   |   dotextok@gmail.com
----------------------------------------------
*/

var mes1 = "Clipboard text in the search line";
var mes2 = "No open documents";
var mes3 = "This script is for InDesign CS6+";
var mes4 = "The script searches the text saved in the clipboard. If part of the material is highlighted, then search in the highlighted area. If the cursor is in the text, then search in the story, starting from the insertion point of the cursor to the end of the article.";
var mes5A = "The PDF file with the description of this program was not found. It's called '";
var mes5B = "' and it should be in the same folder where the script is.";
var mes6 = "It looks like there is no text in the clipboard. Or the contents of the clipboard are not text.";
var mes7 = "Text not found.";

//-----------------
var showPDF = true; // если true, то после сообщения, как определять область поиска, будет выведен PDF-файл Advanced Search Info[lng].pdf

app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll; // см. http://adobeindesign.ru/2008/10/24/restore-ui/
var programTitul = mes1; // "Текст буфера в строке поиска"
if(app.documents.length == 0) {
    alert(mes2, programTitul); // "Нет открытого документа."
    exit();    
    }
if (parseInt (app.version) < 8) {
	alert (mes3, programTitul); // "Этот скрипт предназначен для использования в InDesign CS6+."
    exit();
    }
var sel = app.selection[0];
if ((sel == null) || (sel.constructor.name != "InsertionPoint" && sel.constructor.name != "Paragraph" && sel.constructor.name != "Text" && sel.constructor.name != "TextColumn" && sel.constructor.name != "TextStyleRange")) { // == null
    alert(mes4,programTitul); // "Скрипт ищет в вёрстке сохранённый в буфере текст. Если часть материала выделена, то поиск в выделенной области. Если курсор в тексте, то поиск в материале, начиная от точки вставки курсора до конца статьи."
    if (showPDF) { // showPDF
        app.pdfExportPreferences.properties.viewPDF = true;   
        var myScriptFile = getScriptPath();
        var myScriptFolder = decodeURI(myScriptFile.path);  
      //  var pdfName = "Advanced Search Info.pdf";
        var pdfName = "Advanced Search Info[En].pdf";        
        var infoFile = File(decodeURI(myScriptFolder + "/" + pdfName));
        if (!infoFile.exists) { alert(meso5A + pdfName + mes5B,programTitul); } // "Не найден PDF-файл с описанием этой программы. Он называется '"       "' и должен быть в той же папке, где скрипт."
        else { infoFile.execute(); }
        } // showPDF
    exit(); 
    } // == null
var story = sel.parentStory;    
var space;
if (sel.constructor.name == "InsertionPoint") {
    var startIndex = sel.index;
    var endIndex = story.length-1;
    story.characters.itemByRange(startIndex, endIndex).select();
    space = app.selection[0];
    }
else space = sel;
var startPosition = space.characters[0].index;
var usedZoom = app.activeWindow.zoomPercentage;
var theFrame = app.activeWindow.activePage.textFrames.add({strokeWeight:0});
if (theFrame.strokeWeight != 0) theFrame.strokeWeight = 0;
theFrame.geometricBounds = ["-45mm",0,"-40mm","140mm"];
theFrame.texts[0].appliedParagraphStyle = app.activeDocument.paragraphStyles[0];
theFrame.texts[0].clearOverrides();
theFrame.textFramePreferences.ignoreWrap = true;
theFrame.textFramePreferences.verticalJustification = VerticalJustification.TOP_ALIGN; 
theFrame.texts[0].alignToBaseline = false;
theFrame.texts[0].pointSize = 3;
theFrame.bringToFront();
theFrame.texts[0].contents = "";
theFrame.insertionPoints[0].select();
var erMes = mes6; // "Похоже, что в буфере нет текста. Или содержимое буфера текстом не является."
try {
    app.paste();
    }
catch (e) {
     theFrame.remove();
    alert(erMes + " [1]", programTitul);
    exit();      
    }
if (theFrame.texts[0].length == 0) {
    theFrame.remove();
    alert(erMes + " [2]", programTitul);
    exit();    
    }
///
var notUse = 0;
var newString = "";
// иногда, по непонятной причине, в содержимом буфера после копирования текста из некоторых pdf-файлов сначала идёт перевод строки и пробел, а потом уже тот текст, что был выделен.
for (var i = 0; i < theFrame.parentStory.texts[0].length; i++) { // i++
    if (theFrame.parentStory.texts[0].characters[i].contents == " ") {
        notUse = notUse+1; continue; 
        }
    if (theFrame.parentStory.texts[0].characters[i].contents == "\r") {
        notUse = notUse+1; continue; 
        }    
    if (theFrame.parentStory.texts[0].characters[i].contents == "\n") {
        notUse = notUse+1; continue; 
        }       
    break;
    } // i++
if (notUse > 0) {
    newString = theFrame.parentStory.texts[0].contents.substring(notUse, theFrame.parentStory.texts[0].length);
    theFrame.parentStory.texts[0].contents = newString;
    }
var typoQuotes = app.activeDocument.textPreferences.typographersQuotes;
app.activeDocument.textPreferences.typographersQuotes = false;
////
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\x{003F}"; 
app.changeGrepPreferences.changeTo = "\\?";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\"; 
app.changeGrepPreferences.changeTo = "\\\\"; // это не ошибка, так оформляется в скрипте поиск одиночной наклонной черты. В окне Найти/Заменить для поиска одной наклонной черты их будет стоять две. Объяснение: "экранирование экранирующего символа" -- корявая калька с английского.
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\)"; 
app.changeGrepPreferences.changeTo = "\\)";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\("; 
app.changeGrepPreferences.changeTo = "\\(";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\]"; 
app.changeGrepPreferences.changeTo = "\\]";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\["; 
app.changeGrepPreferences.changeTo = "\\[";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\r"; 
app.changeGrepPreferences.changeTo = "\\r*";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "\\x{00AD}"; 
app.changeGrepPreferences.changeTo = "\\x{00AD}*"; // для обработки переносов в тексте, взятом из PDF
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "(?<=[[:blank:]])[-–—]+(?=\\s)"; // плюс --- для учёта случая, что в вордовском файле тире оформлено как "пробел дефис дефис пробел". Предполагается, что в индизайне текст уже приведён в порядок, прежде чем будет использован этот скрипт
app.changeGrepPreferences.changeTo = "[-–—]";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "[[:blank:]]+"; 
app.changeGrepPreferences.changeTo = "\\h+";
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "[«»„“”\"\'‚‹›‘’]"; 
//app.changeGrepPreferences.changeTo = '\"';  // 05.07.2022  обнаружилось странное поведение при обработке текста этими запросами: если в одном запросе есть обработка кавычки и тире, то запрос не работает.
app.changeGrepPreferences.changeTo = "."; // поэтому для этой задачи поиска на месте любой кавычки будет точка -- по правилам grep это любой символ. Для такого поиска это упрощение вполне приемлемо.
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findGrepPreferences.findWhat = "([\\l\\d])([-‑])([\\l\\d])"; // вариант дефиса между буквами/цифрами -- обычный или неразрывный
app.changeGrepPreferences.changeTo = "$1[-‑]$3"; 
theFrame.parentStory.texts[0].changeGrep();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
app.findChangeGrepOptions.includeFootnotes = true;
newString = theFrame.parentStory.texts[0].contents;
app.findGrepPreferences.findWhat = theFrame.parentStory.texts[0].contents;    
var found = space.findGrep();
theFrame.remove();
///
app.findGrepPreferences = null; 
app.changeGrepPreferences = null;
if (found.length == 0) {
    story.insertionPoints[startPosition].select();
    app.activeWindow.activePage = app.selection[0].parentTextFrames[0].parentPage; 
    app.activeWindow.zoomPercentage = usedZoom;    
    alert(mes7, programTitul); // "Текст не найден."
    app.activeDocument.textPreferences.typographersQuotes = typoQuotes;    
    app.documents[0].select(NothingEnum.nothing);
    exit();
    }
found[0].select();
app.activeWindow.activePage = app.selection[0].parentTextFrames[0].parentPage; 
app.activeWindow.zoomPercentage = 150;
app.activeDocument.textPreferences.typographersQuotes = typoQuotes;
exit();
///////
function getScriptPath() { //getScriptPath()
	try{
		return app.activeScript;
	}
	catch(e){
		return File(e.fileName);
	}
} //getScriptPath()